@@ -1,8 +1,9 @@
 # -*- coding: utf-8 -*-
 from __future__ import unicode_literals
 
-import django.core.validators
+from django.contrib.auth import validators
 from django.db import migrations, models
+from django.utils import six
 
 
 class Migration(migrations.Migration):
@@ -18,11 +19,7 @@ class Migration(migrations.Migration):
             name='username',
             field=models.CharField(
                 error_messages={'unique': 'A user with that username already exists.'}, max_length=30,
-                validators=[django.core.validators.RegexValidator(
-                    '^[\\w.@+-]+$',
-                    'Enter a valid username. This value may contain only letters, numbers and @/./+/-/_ characters.',
-                    'invalid'
-                )],
+                validators=[validators.UnicodeUsernameValidator() if six.PY3 else validators.ASCIIUsernameValidator()],
                 help_text='Required. 30 characters or fewer. Letters, digits and @/./+/-/_ only.',
                 unique=True, verbose_name='username'
             ),
